@@ -1,24 +1,6 @@
-# Copyright (c) 2014-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2014 Google, Inc.
-# Copyright (c) 2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018 Bryce Guinta <bryce.guinta@protonmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 mar-chi-pan <mar.polatoglou@gmail.com>
-# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2020 Andrew Simmons <anjsimmo@gmail.com>
-# Copyright (c) 2020 Andrew Simmons <a.simmons@deakin.edu.au>
-# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Sergei Lebedev <185856+superbobry@users.noreply.github.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/PyCQA/pylint/blob/main/CONTRIBUTORS.txt
 
 import os
 import re
@@ -30,6 +12,7 @@ import astroid
 from pylint.checkers import variables
 from pylint.interfaces import HIGH
 from pylint.testutils import CheckerTestCase, MessageTest, linter, set_config
+from pylint.testutils.reporter_for_tests import GenericTestReporter
 
 REGR_DATA_DIR = str(Path(__file__).parent / ".." / "regrtest_data")
 
@@ -61,7 +44,7 @@ class TestVariablesCheckerWithTearDown(CheckerTestCase):
 
     @set_config(callbacks=("callback_", "_callback"))
     def test_custom_callback_string(self) -> None:
-        """Test the --calbacks option works."""
+        """Test the --callbacks option works."""
         node = astroid.extract_node(
             """
         def callback_one(abc):
@@ -90,7 +73,14 @@ class TestVariablesCheckerWithTearDown(CheckerTestCase):
         )
         with self.assertAddsMessages(
             MessageTest(
-                "unused-argument", node=node["abc"], args="abc", confidence=HIGH
+                "unused-argument",
+                node=node["abc"],
+                args="abc",
+                confidence=HIGH,
+                line=2,
+                col_offset=16,
+                end_line=2,
+                end_col_offset=19,
             )
         ):
             self.checker.visit_functiondef(node)
@@ -104,7 +94,14 @@ class TestVariablesCheckerWithTearDown(CheckerTestCase):
         )
         with self.assertAddsMessages(
             MessageTest(
-                "unused-argument", node=node["abc"], args="abc", confidence=HIGH
+                "unused-argument",
+                node=node["abc"],
+                args="abc",
+                confidence=HIGH,
+                line=2,
+                col_offset=12,
+                end_line=2,
+                end_col_offset=15,
             )
         ):
             self.checker.visit_functiondef(node)
@@ -118,7 +115,15 @@ class TestVariablesCheckerWithTearDown(CheckerTestCase):
         """
         )
         with self.assertAddsMessages(
-            MessageTest("redefined-builtin", node=node.body[0], args="open")
+            MessageTest(
+                "redefined-builtin",
+                node=node.body[0],
+                args="open",
+                line=2,
+                col_offset=0,
+                end_line=2,
+                end_col_offset=32,
+            )
         ):
             self.checker.visit_module(node)
 
@@ -176,7 +181,8 @@ class TestVariablesCheckerWithTearDown(CheckerTestCase):
     @set_config(ignored_argument_names=re.compile("arg"))
     def test_ignored_argument_names_no_message(self) -> None:
         """Make sure is_ignored_argument_names properly ignores
-        function arguments"""
+        function arguments
+        """
         node = astroid.parse(
             """
         def fooby(arg):
@@ -207,6 +213,7 @@ class TestMissingSubmodule(CheckerTestCase):
         sys.path.insert(0, REGR_DATA_DIR)
         try:
             linter.check([os.path.join(REGR_DATA_DIR, "package_all")])
+            assert isinstance(linter.reporter, GenericTestReporter)
             got = linter.reporter.finalize().strip()
             assert got == "E:  3: Undefined variable name 'missing' in __all__"
         finally:
